{-# LANGUAGE RecordWildCards, RankNTypes, DeriveDataTypeable, DeriveGeneric, TemplateHaskell #-}

{-|

Module      : Compiler

Description : Functions that compile instructions to the internal representation

Copyright   :  © 2015 W. Garrett Mitchener

License     : GPL-3

Maintainer  : garrett.mitchener@gmail.com

Stability   : experimental

Portability : non-portable (GHC, POSIX)

-}

module Compiler (
  -- * Main interface
  ProgramInformation(..),
  ReactionProgram,
  compileUnchecked
  ) where

import Control.Monad
import Control.Monad.ST

import Data.Aeson.TH
import Data.Data
import Data.Either
import qualified Data.IntMap as IM
import qualified Data.IntSet as IS
import GHC.Generics

import Instruction
import MVM
import VMViaPrim

compiler_vcid :: String
compiler_vcid = "$Id: Compiler.hs,v 002890d5829b 2017-01-19 18:09:45Z garrett $"

-- | Information assembled during compilation

data ProgramInformation = ProgramInformation {
  instructions :: [Instruction],
  -- ^ instructions used to build this machine
  reactions :: [Reaction],
  -- ^ reactions extracted from the instructions
  patternsReferenced :: IS.IntSet
  -- ^ set of patterns referenced by the program
  } deriving (Read, Show, Eq, Ord, Data, Typeable, Generic)


-- | Compile the given list of 'Instructions', without checking
-- for mistakes.

compileUnchecked ::
  [Instruction] -- ^ the program
  -> (ProgramInformation, ReactionProgram)

compileUnchecked instructions =
  let
    reactions =
      concatMap splitInstruction (zip [0..] instructions)

    basicReactions = [(sPat, thr, tPat, delta)
                      | Reaction{..} <- reactions]

    compiledProgram = primCompile basicReactions

    patternsReferenced =
      IS.fromList $ concat [[sPat, tPat] | Reaction{..} <- reactions]
  in
   (ProgramInformation{..}, compiledProgram)

$(deriveJSON defaultOptions ''ProgramInformation)
