{-# LANGUAGE RecordWildCards, NamedFieldPuns, DeriveDataTypeable, DeriveGeneric, ScopedTypeVariables, ConstraintKinds, FlexibleContexts, OverloadedStrings, Arrows #-}

{-|
Module      : CrossHammingDistances

Description : Extract useful genes for doing phylogenetics experiments

Copyright   : © 2015 W. Garrett Mitchener

License     : GPL-3

Maintainer  : garrett.mitchener@gmail.com

Stability   : experimental

Portability : non-portable (GHC, POSIX)

Given two file names on the command line, A and B, read them as
01-strings.  For each string in file A, compute the Hamming
distance between that and each string in B.  The highest one is
the score for that string.  Output the top n scores.

-}

module Main where

import Control.Applicative
import Control.Monad

import Data.List

import Options.Applicative
import Options.Applicative.Arrows

import System.FilePath
import System.IO

import BitString

crossHammingDistances_vcid :: String
crossHammingDistances_vcid = "$Id: CrossHammingDistances.hs,v 5a02784b68c3 2017-01-19 18:10:41Z garrett $"

data CrossHammingJob =
  CrossHammingJob {
    aFilePath :: FilePath,
    bFilePath :: FilePath
    }
  deriving (Show)

chjobParser :: ParserInfo CrossHammingJob
chjobParser =
  (info chjobOpts
   (progDesc "\
\Given two file names on the command line, A and B, read them as \
\01-strings.  For each string in file A, compute the Hamming \
\distance between that and each string in B.  The best one (lowest) is \
\the score for that string.  Output those scores."))


chjobOpts :: Parser CrossHammingJob
chjobOpts = runA $ proc () -> do
  aFilePath <-
    (asA $ argument str
     (metavar "FILE-A"
      <> help "First file")
    ) -< ()
  bFilePath <-
    (asA $ argument str
     (metavar "FILE-B"
      <> help "Second file")
    ) -< ()
  returnA -< CrossHammingJob{..}



read01StringFile filePath = do
  fileData <- readFile filePath
  let fileLines = lines fileData
  return (map bsFrom01String fileLines)

main = do
  CrossHammingJob{..} <- execParser chjobParser

  aStrings <- read01StringFile aFilePath
  bStrings <- read01StringFile bFilePath
  let hammingDistances = sort [
        minimum [bsHammingDistance a b | b <- bStrings]
        | a <- aStrings]
  forM_ hammingDistances $ \n -> do
    print n
