{-# LANGUAGE NoMonomorphismRestriction, RankNTypes, ScopedTypeVariables #-}

module Logging where

import Control.Monad.IO.Class
import qualified Control.Exception as E
import System.Log.Logger

logging_vcid :: String
logging_vcid = "$Id: Logging.hs,v 5a02784b68c3 2017-01-19 18:10:41Z garrett $"

loggerName :: String
loggerName = "ALifeUM7"

type LogMessager = forall m. MonadIO m => String -> m ()

msgDebug :: LogMessager
msgDebug msg = liftIO $ debugM loggerName msg

msgInfo :: LogMessager
msgInfo msg = liftIO $ infoM loggerName msg

msgWarning :: LogMessager
msgWarning msg = liftIO $ warningM loggerName msg


msgError :: LogMessager
msgError msg = liftIO $ errorM loggerName msg

msgExc :: E.Exception e => e -> LogMessager
msgExc e msg = do
  msgError msg
  E.throw e
  

-- | Permform @axn@.  If an exception is raised, call 'msgError'
-- with a message indicating the @place@ and displaying the
-- exception.
excMsg :: String -- ^ @place@
          -> IO a -- ^ @axn@
          -> IO a
excMsg place axn = do
  mRes <- E.try axn
  case mRes of
    Right result ->
      return result
    Left (e :: E.SomeException) -> do
      msgError ("At " ++ place ++ " caught " ++ show e)
      E.throwIO e

