{-# LANGUAGE RecordWildCards, NamedFieldPuns, DeriveDataTypeable, DeriveGeneric, TemplateHaskell, StandaloneDeriving, FlexibleContexts, UndecidableInstances  #-}

{-|

Module: PopulationSpec

Description: Data structures for specifying the parts of a run
             that only pertain to the population structure, not
             the problem being solved

Copyright   : © W. Garrett Mitchener 2015

License     : GPL-3

Maintainer  : garrett.mitchener@gmail.com

Stability   : experimental

Portability : POSIX


-}

module PopulationSpec where

import GHC.Generics

import Data.Aeson
import Data.Data
import Data.DeriveTH
import Data.Typeable

import Database
import Evolution
import Genome

populationSpec_vcid :: String
populationSpec_vcid = "$Id: PopulationSpec.hs,v 5a02784b68c3 2017-01-19 18:10:41Z garrett $"

data PopulationSpec gs =
  PopulationSpec {
    populationSize :: Int,
    databaseSpec :: DatabaseSpec,
    genomeSpec :: gs,
    selectionSpec :: SelectionSpec,
    alignmentSpec :: AlignmentSpec gs
    }

deriving instance (Read gs, Read (AlignmentSpec gs)) => Read (PopulationSpec gs)
deriving instance (Show gs, Show (AlignmentSpec gs)) => Show (PopulationSpec gs)
deriving instance (Data gs, Data (AlignmentSpec gs)) => Data (PopulationSpec gs)
deriving instance Typeable PopulationSpec

