{-# LANGUAGE MultiParamTypeClasses, FlexibleInstances, TemplateHaskell, DeriveDataTypeable, DeriveGeneric #-}

{-|

Module: PredicateTools

Description : Utilities for dealing with predicates in a monad.

Copyright   : © 2015 W. Garrett Mitchener

License     : GPL-3

Maintainer  : garrett.mitchener@gmail.com

Stability   : experimental

Portability : POSIX

-}

module PredicateTools (
  MPredicate(..),
  CompoundMPredicate(..),
  ) where

import GHC.Generics

import Control.DeepSeq

import Data.Aeson.TH
import Data.Binary
import Data.DeriveTH
import Data.Data
import Data.Typeable

import AesonUtil

predicateTools_vcid :: String
predicateTools_vcid = "$Id: PredicateTools.hs,v 5a02784b68c3 2017-01-19 18:10:41Z garrett $"

-- | Class for objects that can check whether something is true
-- in the context of a monad.

class MPredicate m p t where
  check ::
    t -- ^ the object to check
    -> p -- ^ the predicate to apply to it
    -> m Bool

-- | A composite monadic predicate
data CompoundMPredicate p =
  Always
  | Never
  | Check p
  | AllOf [CompoundMPredicate p]
  | AnyOf [CompoundMPredicate p]
  | NoneOf [CompoundMPredicate p]
    deriving (Read, Show, Eq, Ord, Typeable, Data, Generic)

$(derive makeNFData ''CompoundMPredicate)
$(derive makeBinary ''CompoundMPredicate)
$(deriveJSON myJSONOpts ''CompoundMPredicate)

instance
  (Monad m, MPredicate m p t)
  => MPredicate m (CompoundMPredicate p) t where

    check t (Check p) =  check t p

    check _ Always = return True

    check _ Never = return False

    check t (AllOf ps) = do
      checks <- mapM (check t) ps
      return (and checks)

    check t (AnyOf ps) = do
      checks <- mapM (check t) ps
      return (or checks)

    check t (NoneOf ps) = do
      checks <- mapM (check t) ps
      return (not (or checks))
