{-# LANGUAGE RecordWildCards, NamedFieldPuns, ScopedTypeVariables, DeriveDataTypeable, DeriveGeneric, TemplateHaskell #-}

{-|

Module: QueryRequest

Description : Ways to query agents

Copyright   : © 2015 W. Garrett Mitchener

License     : GPL-3

Maintainer  : garrett.mitchener@gmail.com

Stability   : experimental

Portability : non-portable (GHC, POSIX)

Possible ways of querying an agent.  These datatypes are
declared in their own module to avoid name clashes with
"QueryResult".

-}

module QueryRequest where

import GHC.Generics

import Data.Aeson
import Data.Aeson.TH
import Data.Data
import Data.Typeable

import AesonUtil

queryRequest_vcid :: String
queryRequest_vcid = "$Id: QueryRequest.hs,v 5a02784b68c3 2017-01-19 18:10:41Z garrett $"

-- | Information that can be extracted from a single 'Agent';
-- These are parellel to 'QueryResult.AgentResult'.
data AgentRequest =
  JustAgent
  | Summary
  | Export
  | Program
  | GenomeDetails
  | Efficacious
  | FullDetails
  | RatingInputs
  | ReactionNetwork
  deriving (Read, Show, Eq, Ord, Data, Typeable, Generic)

$(deriveJSON myJSONOpts ''AgentRequest)
