{-# LANGUAGE DefaultSignatures, MultiParamTypeClasses #-}

{-|
Module      : Snip

Description : Generating reports from agent queries

Copyright   : © 2015 W. Garrett Mitchener

License     : GPL-3

Maintainer  : garrett.mitchener@gmail.com

Stability   : experimental

Portability : non-portable (GHC, POSIX)

Type class for things that can be made into snips for nice spiffy
Pandoc documents.

-}

module Snip where

import Data.Aeson (ToJSON(..))
import qualified Data.ByteString as BS
import qualified Data.ByteString.UTF8 as U
import qualified Data.Yaml as Y
import Text.Pandoc
import Text.Pandoc.Builder

snip_vcid :: String
snip_vcid = "$Id: Snip.hs,v 5a02784b68c3 2017-01-19 18:10:41Z garrett $"

-- | A 'Snip' object can be converted to Pandoc 'Blocks'.  The
-- @opt@ object is a way to specify options.
class Snip opt t where

  toBlocks :: opt -> t -> Blocks

  -- | The default implementation is to convert the object to
  -- JSON and write it in YAML as a code block.
  default toBlocks :: ToJSON t => opt -> t -> Blocks
  toBlocks _ x =
    codeBlock $ U.toString $ Y.encode x


-- | A 'Sniplet' object can be converted to Pandoc 'Inlines'.
-- The @opt@ object is a way to specify options.
class Sniplet opt t where

  toInlines :: opt -> t -> Inlines

  -- | The default implementation is to convert the object to
  -- JSON and write that as a line of code.
  default toInlines :: ToJSON t => opt -> t -> Inlines
  toInlines _ x =
    code $ U.toString $ Y.encode x
