{-# LANGUAGE RecordWildCards, NamedFieldPuns, FlexibleContexts, DeriveDataTypeable, DeriveGeneric, TemplateHaskell #-}

{-|

Module: StyledNetwork

Description: Tools for building visual representations of gene networks

Copyright: © 2016 W. Garrett Mitchener

License     : GPL-3

Maintainer  : garrett.mitchener@gmail.com

Stability   : experimental

Portability : GHC

-}

module StyledNetwork where

import GHC.Generics

import Data.Aeson
import Data.Aeson.TH
import Data.Data
import Data.Typeable

import AesonUtil

styledNetwork_vcid :: String
styledNetwork_vcid = "$Id: StyledNetwork.hs,v 5a02784b68c3 2017-01-19 18:10:41Z garrett $"

data StyledNode =
  StyledNode {
    nodeKey :: String,
    nodeLabel :: String,
    nodeStyle :: NodeStyle
    } deriving (Read, Show, Eq, Ord, Data, Typeable, Generic)


data NodeStyle =
  OrdinaryNode
  | InputSourceNode
  | BackgroundInputTargetNode
  | InputTargetNode
  | OutputSourceNode
  | OutputTargetNode
  | SynapseSourceNode
  | SynapseTargetNode
  | StopNode
  | ReducedNode
  deriving (Read, Show, Eq, Ord, Enum, Data, Typeable, Generic)

data StyledEdge =
  StyledEdge {
    edgeKey :: String,
    fromKey :: String,
    toKey :: String,
    edgeLabel :: String,
    edgeStyle :: EdgeStyle
    } deriving (Read, Show, Eq, Ord, Data, Typeable, Generic)

data EdgeStyle =
  ActivationEdge
  | InhibitionEdge
  deriving (Read, Show, Eq, Ord, Enum, Data, Typeable, Generic)

data StyledNetwork =
  StyledNetwork {
    networkKey :: String,
    networkLabel :: String,
    nodes :: [StyledNode],
    edges :: [StyledEdge],
    subnetworks :: [StyledNetwork]
    } deriving (Read, Show, Eq, Ord, Data, Typeable, Generic)


$(deriveJSON myJSONOpts ''StyledNode)
$(deriveJSON myJSONOpts ''NodeStyle)
$(deriveJSON myJSONOpts ''StyledEdge)
$(deriveJSON myJSONOpts ''EdgeStyle)
$(deriveJSON myJSONOpts ''StyledNetwork)

