module BinaryReflectedGrayCode where

import Data.Bits
import Data.Word
import Data.Int
import Foreign.C.Types

binaryReflectedGrayCode_vcid :: String
binaryReflectedGrayCode_vcid = "$Id: BinaryReflectedGrayCode.hs,v 1607c537d3ac 2017-01-19 18:10:07Z garrett $"

binaryToReflectedGray :: (Show n, Eq n, Num n, Ord n, Bits n) =>
                         n -> n
                         
binaryToReflectedGray n | n >= 0 = n `xor` (n `shiftR` 1)
                        | otherwise = error ("binaryToReflectedGray: n must be >= 0; given n = " ++ show n)
                                      
{-# SPECIALIZE INLINE binaryToReflectedGray :: Integer -> Integer #-}
{-# SPECIALIZE INLINE binaryToReflectedGray :: Word -> Word #-}
{-# SPECIALIZE INLINE binaryToReflectedGray :: Word8 -> Word8 #-}
{-# SPECIALIZE INLINE binaryToReflectedGray :: Word16 -> Word16 #-}
{-# SPECIALIZE INLINE binaryToReflectedGray :: Word32 -> Word32 #-}
{-# SPECIALIZE INLINE binaryToReflectedGray :: Word64 -> Word64 #-}
{-# SPECIALIZE INLINE binaryToReflectedGray :: Int -> Int #-}
{-# SPECIALIZE INLINE binaryToReflectedGray :: Int8 -> Int8 #-}
{-# SPECIALIZE INLINE binaryToReflectedGray :: Int16 -> Int16 #-}
{-# SPECIALIZE INLINE binaryToReflectedGray :: Int32 -> Int32 #-}
{-# SPECIALIZE INLINE binaryToReflectedGray :: Int64 -> Int64 #-}
{-# SPECIALIZE INLINE binaryToReflectedGray :: CInt -> CInt #-}
{-# SPECIALIZE INLINE binaryToReflectedGray :: CUInt -> CUInt #-}
{-# SPECIALIZE INLINE binaryToReflectedGray :: CShort -> CShort #-}
{-# SPECIALIZE INLINE binaryToReflectedGray :: CUShort -> CUShort #-}
{-# SPECIALIZE INLINE binaryToReflectedGray :: CLong -> CLong #-}
{-# SPECIALIZE INLINE binaryToReflectedGray :: CULong -> CULong #-}
{-# SPECIALIZE INLINE binaryToReflectedGray :: CLLong -> CLLong #-}
{-# SPECIALIZE INLINE binaryToReflectedGray :: CULLong -> CULLong #-}
{-# SPECIALIZE INLINE binaryToReflectedGray :: CSize -> CSize #-}
{-# SPECIALIZE INLINE binaryToReflectedGray :: CChar -> CChar #-}
{-# SPECIALIZE INLINE binaryToReflectedGray :: CSChar -> CSChar #-}
{-# SPECIALIZE INLINE binaryToReflectedGray :: CUChar -> CUChar #-}
{-# SPECIALIZE INLINE binaryToReflectedGray :: CWchar -> CWchar #-}

reflectedGrayToBinary :: (Show n, Eq n, Num n, Ord n, Bits n) =>
                         n -> n
reflectedGrayToBinary n 
  | n >= 0 = helper n (n `shiftR` 1)
  | otherwise = error ("reflectedGrayToBinary: n must be >= 0; given n = " ++ show n)
    
  where
    helper m mask | mask == 0  = m
                  | otherwise  = helper (m `xor` mask) (mask `shiftR` 1)

{-# SPECIALIZE reflectedGrayToBinary :: Integer -> Integer #-}
{-# SPECIALIZE reflectedGrayToBinary :: Word -> Word #-}
{-# SPECIALIZE reflectedGrayToBinary :: Word8 -> Word8 #-}
{-# SPECIALIZE reflectedGrayToBinary :: Word16 -> Word16 #-}
{-# SPECIALIZE reflectedGrayToBinary :: Word32 -> Word32 #-}
{-# SPECIALIZE reflectedGrayToBinary :: Word64 -> Word64 #-}
{-# SPECIALIZE reflectedGrayToBinary :: Int -> Int #-}
{-# SPECIALIZE reflectedGrayToBinary :: Int8 -> Int8 #-}
{-# SPECIALIZE reflectedGrayToBinary :: Int16 -> Int16 #-}
{-# SPECIALIZE reflectedGrayToBinary :: Int32 -> Int32 #-}
{-# SPECIALIZE reflectedGrayToBinary :: Int64 -> Int64 #-}
{-# SPECIALIZE reflectedGrayToBinary :: CInt -> CInt #-}
{-# SPECIALIZE reflectedGrayToBinary :: CUInt -> CUInt #-}
{-# SPECIALIZE reflectedGrayToBinary :: CShort -> CShort #-}
{-# SPECIALIZE reflectedGrayToBinary :: CUShort -> CUShort #-}
{-# SPECIALIZE reflectedGrayToBinary :: CLong -> CLong #-}
{-# SPECIALIZE reflectedGrayToBinary :: CULong -> CULong #-}
{-# SPECIALIZE reflectedGrayToBinary :: CLLong -> CLLong #-}
{-# SPECIALIZE reflectedGrayToBinary :: CULLong -> CULLong #-}
{-# SPECIALIZE reflectedGrayToBinary :: CSize -> CSize #-}
{-# SPECIALIZE reflectedGrayToBinary :: CChar -> CChar #-}
{-# SPECIALIZE reflectedGrayToBinary :: CSChar -> CSChar #-}
{-# SPECIALIZE reflectedGrayToBinary :: CUChar -> CUChar #-}
{-# SPECIALIZE reflectedGrayToBinary :: CWchar -> CWchar #-}
