{-# LANGUAGE ForeignFunctionInterface, EmptyDataDecls, ScopedTypeVariables #-}

module CSFMT (Generator, newGeneratorFromInt, newGeneratorFromWords,
              randomUnitDouble, randomInt)
where
  
import Foreign hiding (unsafePerformIO)
import Foreign.C
import Control.Monad.ST hiding (unsafeIOToST)
import Control.Monad.ST.Unsafe
import System.IO
import System.IO.Unsafe

csfmt_vcid :: String
csfmt_vcid = "$Id: CSFMT.hs,v 5996a286e558 2015-05-26 13:34:15Z garrett $"

data CSFMT_prim

type CSFMT = Ptr CSFMT_prim

type Generator = ForeignPtr CSFMT_prim

foreign import ccall unsafe "CachingSFMT.h csfmt_new" 
  csfmt_new :: CInt -> IO CSFMT

foreign import ccall unsafe "CachingSFMT.h csfmt_newFromWords" 
  csfmt_newFromWords :: Ptr Word32 -> CInt -> IO CSFMT

foreign import ccall unsafe "CachingSFMT.h csfmt_genrand_close_open" 
  csfmt_genrand_close_open :: CSFMT -> IO Double

foreign import ccall unsafe "CachingSFMT.h csfmt_finalize" 
  csfmt_finalize :: CSFMT -> IO ()

foreign import ccall unsafe "CachingSFMT.h &csfmt_finalize" 
  csfmt_finalize_p :: FunPtr (CSFMT -> IO ())

{-# NOINLINE newGeneratorFromInt #-}

newGeneratorFromInt :: Int -> Generator
newGeneratorFromInt seed = unsafePerformIO $ do
  pGen <- csfmt_new (fromIntegral seed)
  newForeignPtr csfmt_finalize_p pGen

{-# NOINLINE newGeneratorFromWords #-}

newGeneratorFromWords :: [Word32] -> Generator
newGeneratorFromWords seeds = unsafePerformIO $ do
  pGen <- withArrayLen seeds $ \nWords ptr ->
    csfmt_newFromWords ptr (fromIntegral nWords)
  newForeignPtr csfmt_finalize_p pGen

{-# NOINLINE randomUnitDouble #-}  
{- | Create a random double in the range [0,1) -}
randomUnitDouble :: Generator -> ST s Double
randomUnitDouble fpGen =
  unsafeIOToST $ withForeignPtr fpGen $ \pGen ->
     csfmt_genrand_close_open pGen

randomInt :: forall s n. (Integral n, Bounded n) => 
             Generator -> ST s n
randomInt g = 
  let 
    a = fromIntegral (minBound :: n)
    b = fromIntegral (maxBound :: n)
    spread = b + 1 - a
  in do
    x <- randomUnitDouble g
    let n = floor (a + spread * x)
    n `seq` return n

testNoFinalizer = do
  hPutStrLn stderr "Start"
  gen <- csfmt_new 12345
  hPutStrLn stderr "Generator initialized"
  x <- csfmt_genrand_close_open gen
  hPutStrLn stderr "Generator run"
  putStrLn (show x)
  putStrLn "Ok"  
