{- | Various little recipes for using parallel evaluation -}

module ParUtil where

import Control.Parallel.Strategies
import Control.DeepSeq
import Data.List.Split

parUtil_vcid :: String
parUtil_vcid = "$Id: ParUtil.hs,v d8e4380c1184 2017-01-19 18:11:01Z garrett $"

-- | Return the list, but it's evaluated in parallel with 'deepSeq'
-- using a "throttling" mechanism to keep from running out of
-- sparks.  This was suggested at
-- http://www.haskell.org/haskellwiki/ThreadScope_Tour/Spark2
parListDeep :: NFData a => [a] -> [a]
parListDeep xs =
  force (xs `using` parBuffer 64 rdeepseq)

{-# INLINABLE parListDeep #-}


-- | Similar to 'parListDeep' but the list is split into chunks,
-- and those are evaluated in parallel.

parListDeepInChunks ::
  NFData a =>
  Int -- ^ items per chunk
  -> [a]
  -> [a]

parListDeepInChunks chunkSize xs
  | (chunkSize > 1) =
    parListDeep (chunksOf chunkSize xs) `seq` xs
  | otherwise =
      parListDeep xs

{-# INLINABLE parListDeepInChunks #-}

  
--  force (xs `using` parList rdeepseq)


