{-# LANGUAGE MultiParamTypeClasses, FlexibleInstances, TypeFamilies, ScopedTypeVariables, RankNTypes #-}

{- |

Module: Snapshot

Description: How to record the state of a dynamic object

Copyright: &#xA9; 2015 W. Garrett Mitchener

License     : GPL-3

Maintainer  : garrett.mitchener@gmail.com

Stability   : experimental

Portability : Portable

-}

module Snapshot where

import Control.Monad.ST
import Data.STRef

class Monad m => Snapshot m t where
  type Memento t :: *

  snapshot :: t -> m (Memento t)


instance
  (Monad m,
   Snapshot m t)
  => Snapshot m [t]   where
  type Memento [t] = [Memento t]
  snapshot = mapM snapshot

instance
  (Monad m,
   Snapshot m t1,
   Snapshot m t2)
  => Snapshot m (t1, t2)   where
  type Memento (t1, t2) = (Memento t1, Memento t2)
  snapshot (x1, x2) = do
    m1 <- snapshot x1
    m2 <- snapshot x2
    return (m1, m2)


instance Snapshot (ST s) (STRef s t) where
  type Memento (STRef s t) = t

  snapshot ref =
    readSTRef ref
